import { OnLoadArgs, OnLoadResult, Plugin } from 'esbuild';
import { Agent } from 'http';
import makeFetchHappen, { FetchOptions } from 'make-fetch-happen';

type Response = Awaited<ReturnType<typeof makeFetchHappen>>;

interface HttpPluginOptions {
    /** Return `true` to filter in paths to be handled by this plugin. */
    filter?: RegExp | ((url: string) => boolean);
    /** Custom http.Agent. */
    agent?: Agent | null;
    /** Custom url scheme, example: `{ unpkg: 'https://unpkg.com/?module' }`. */
    schemes?: Record<string, string>;
    /** Cache request results. */
    cache?: Map<string, string | Uint8Array>;
    /** onfetch(console.log) */
    onfetch?: (url: string) => void;
    /** Passed to `make-fetch-happen`, the `cachePath` and `agent` is already set. */
    fetchOptions?: FetchOptions;
    /**
     * Custom load result, the `contents` is the fetched data.
     *
     * By default this plugin passes `loader: 'default'` to esbuild, which means
     * it relies on a correct file extension to determine the loader.
     *
     * If in any cases you find the file extension is wrong or missing, you can
     * set this option to override the default behavior.
     * ```js
     * onload(url, contents) {
     *   return { contents, loader: 'default' }
     * }
     * ```
     * Advanced usage:
     * ```js
     * onload(url, contents, args, response) {
     *   // if fetched from cache, the `response` will be null.
     * }
     * ```
     */
    onload?: (url: string, contents: string | Uint8Array, args: OnLoadArgs, response: Response | null) => OnLoadResult | Promise<OnLoadResult | null | undefined> | null | undefined;
}
/** https://esbuild.github.io/plugins/#http-plugin */
declare function http({ filter, agent, schemes, cache, onfetch, fetchOptions, onload, }?: HttpPluginOptions): Plugin;
declare const default_schemes: Record<string, string>;

export { HttpPluginOptions, http as default, default_schemes, http };
